package cn.com.ty.lift.system.user.service.impl;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.system.entity.Role;
import cn.com.ty.lift.system.entity.UserRole;
import cn.com.ty.lift.system.entity.model.RoleRequest;
import cn.com.ty.lift.system.mapper.RoleMapper;
import cn.com.ty.lift.system.user.service.IRoleMenuService;
import cn.com.ty.lift.system.user.service.IRoleService;
import cn.com.ty.lift.system.user.service.IUserRoleService;
import cn.com.xwy.boot.web.dto.RestResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author huangyuan
 * @date 2019-12-01
 * @description 角色接口实现类
 */
@Service
public class RoleService implements IRoleService {

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private IRoleMenuService roleMenuService;

    @Autowired
    private IUserRoleService userRoleService;

    @Override
    public RestResponse list(RoleRequest roleRequest) {
        return roleMapper.getByRoleRequest(roleRequest);
    }

    @Override
    @Transactional
    public RestResponse addRole(RoleRequest roleRequest) {
        Role role = new Role(roleRequest);
        role.setCreateTime(new Date());
        //保存角色信息
        int insertCount = roleMapper.insert(role);
        //保存角色菜单关联关系
        boolean saveBatchFlag = roleMenuService.saveRoleMenu(role.getId(), roleRequest.getMenuIds());
        if (insertCount == 0 || !saveBatchFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "保存角色信息失败");
        }
        return RestResponse.ok(role, ApiConstants.RESULT_SUCCESS, "保存角色信息成功");
    }

    @Override
    @Transactional
    public RestResponse updateRole(RoleRequest roleRequest) {
        Role role = new Role(roleRequest);
        role.setUpdateTime(new Date());
        //更新角色信息
        int updateCount = roleMapper.updateByPrimaryKey(role);
        //删除角色原来关联的菜单信息
        boolean deleteFlag = roleMenuService.deleteRoleMenu(role.getId());
        //保存角色关联的新的菜单信息
        boolean saveBatchFlag = roleMenuService.saveRoleMenu(role.getId(), roleRequest.getMenuIds());
        if (updateCount == 0 || !deleteFlag || !saveBatchFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "更新角色信息失败");
        }
        return RestResponse.ok(role, ApiConstants.RESULT_SUCCESS, "更新角色信息成功");
    }

    @Override
    @Transactional
    public RestResponse deleteRole(RoleRequest roleRequest) {
        Long roleId = roleRequest.getId();
        //判断角色有没有分配给用户
        List<UserRole> userRoleList = userRoleService.getByRoleId(roleId);
        if(userRoleList != null && userRoleList.size() > 0){
            return RestResponse.error(ApiConstants.RESULT_ERROR, "角色已关联用户，无法删除");
        }
        Role role = roleMapper.selectByPrimaryKey(roleId);
        role.setDeleteUserId(roleRequest.getDeleteUserId());
        role.setDeleteTime(new Date());
        role.setDeleteFlag(ApiConstants.DELETE_YES);
        int updateCount = roleMapper.updateByPrimaryKey(role);
        //删除角色菜单关联信息
        boolean deleteFlag = roleMenuService.deleteRoleMenu(roleId);
        if (updateCount == 0 || !deleteFlag) {
            return RestResponse.error(ApiConstants.RESULT_ERROR, "删除失败");
        }
        return RestResponse.ok(null, ApiConstants.RESULT_SUCCESS, "删除成功");
    }
}
